var fs = require('fs');
var filePath = null;
// Now that we have the old messages, open the file for appending new ones.
// 'a+' - Open file for reading and appending.
// The file is created if it does not exist.
var ENCODING = 'utf8';
var writeOptions = { flags: 'a+', encoding: ENCODING, mode: 0766 };
var onLoad = null;
var messages = [];
var writeStream = null;
var fd = null;
var streamReady = false;
var nextWriteIndex = 0;

function onFileOpenForAppending ( newFd ) {
  fd = newFd;
  streamReady = true;
  console.log(filePath+" open for appending with file descriptor: ", fd);
  onLoad( null, messages );
}// end of function onFileOpenForAppending

function onFileRead ( err, data ) {
  if ( err ) {
    onLoad(err);
    return;
  }
  console.log("Read from "+filePath+" complete.");
  // Since the log is a comma-separated list of stringified messages,
  // We wrap it in this array and surrounding object before parsing the whole thing.
  // var messageObject = JSON.parse( '{ messages: ['+data+'] }' );
  // messages = messageObject.messages;
  messages = JSON.parse('['+data+']');
  nextWriteIndex = messages.length;
  console.log("retrieved "+messages.length+" messages");
  // Now that we have the old messages, open the file for appending new ones.
  // 'a+' - Open file for reading and appending.
  // The file is created if it does not exist.
  writeStream = fs.createWriteStream( filePath, writeOptions );
  writeStream.on('open',onFileOpenForAppending);
}// end of function onFileRead

// newFilePath should be a string.
// newOnLoad should take err and messages, an array of message objects.
exports.load = function ( newFilePath, newOnLoad ) {
  
  filePath = newFilePath;
  onLoad = newOnLoad;
  // First, retrieve the messages from the previous run.
  fs.readFile( filePath, { encoding:ENCODING , flag:'r' }, onFileRead );
  
};// end of exports.load

exports.get = function ( startIndex ) {
  return messages.slice(startIndex);
};// end of exports.get

// Notice that calling save initiates a cycle wherein writeIfReady writes and
// calls onFlush when it is done, at which time onFlush calls writeIfReady.
// This continues until writeIfReady has nothing more to write, that is,
// when nextWriteIndex has caught up to messages.length.
// If we call save again while this cycle is in progress, the new message gets
// pushed to the end of messages and will be written once all messages before it
// have been written.

function onFlush () {
  streamReady = true;
  writeIfReady();
}// end of function onFlush

function writeIfReady () {
  if ( streamReady && nextWriteIndex < messages.length ) {
    // After the first message, put a comma, then a new line after the previous
    // message to separate it from the one we are writing now.
    var separator = (nextWriteIndex > 0) ? ',\n':'';
    var messageString = separator + JSON.stringify(messages[nextWriteIndex]);
    nextWriteIndex++;
    streamReady = writeStream.write( messageString, ENCODING, onFlush );
  }
}// end of function writeIfReady

exports.save = function ( message ) {
  messages.push(message);
  writeIfReady();
}// end of exports.save

exports.end = function () {
  fs.closeSync(fd);
}// end of exports.end