
// We should use a consistent format for representing date-times throughout the application.
function makeTimeText (timestamp) {
  return ( new Date(timestamp) ).toISOString();
}// end of function makeTimeText

var AT_END = -1;

function addCellAndOption ( row, select, message, key, makeText ) {
  
  if ( !message.hasOwnProperty(key) ) {
    // If this key is not present in the message, insert a blank cell as a placeholder.
    row.insertCell(AT_END);
    return;
  }
  var value = message[key];
  var text = value;
  if ( makeText !== null ) {
    text = makeText(value);
  }
  row.insertCell(AT_END).textContent = text;
  var valueIsNew = true;
  for ( var index = 0; index < select.options.length; index++ ) {
    // In this case, we do want to allow typecasting of numbers to strings.
    if ( select.options[index].value == value ) {
      valueIsNew = false;
    }
  }
  if ( valueIsNew ) {
    // true here indicates that edits with this value should be selected (shown) by default.
    // The first true is for defaultSelected, the second for selected.
    select.add(  new Option( text, value, true, true )  );
  }
  
}// end of function addOption

function setUpEditHistoryTable () {
  
  var editTable = document.getElementById('editTable');
  var clientIdSelect = document.getElementById('clientIdSelect');
  var actionSelect = document.getElementById('actionSelect');
  var timestampSelect = document.getElementById('timestampSelect');
  var viewIdSelect = document.getElementById('viewIdSelect');
  var nodeIdSelect = document.getElementById('nodeIdSelect');
  var edgeIdSelect = document.getElementById('edgeIdSelect');
  var userNameSelect = document.getElementById('userNameSelect');
  var textSelect = document.getElementById('textSelect');
  var urlSelect = document.getElementById('urlSelect');
  var xSelect = document.getElementById('xSelect');
  var ySelect = document.getElementById('ySelect');
  var subjectIdSelect = document.getElementById('subjectIdSelect');
  var predicateIdSelect = document.getElementById('predicateIdSelect');
  var objectIdSelect = document.getElementById('objectIdSelect');
  var selects = [clientIdSelect,actionSelect,timestampSelect,viewIdSelect,nodeIdSelect,edgeIdSelect,userNameSelect,textSelect,urlSelect,xSelect,ySelect,subjectIdSelect,predicateIdSelect,objectIdSelect];
  
  function setRowDisplayed ( row ) {
    // Check every column to see whether the cell value is one of the ones selected.
    // console.log( " setting visibility of row ", row );
    // console.log( "cells: "+row.cells.length+" vs. selects: "+selects.length );
    for ( var selectIndex = 0; selectIndex < selects.length; selectIndex++ ) {
      var select = selects[selectIndex];
      var cell = row.cells[selectIndex];
      // Look for the value in the cell among the options in the select for this column.
      for ( var optionIndex = 0; optionIndex < select.options.length; optionIndex++ ) {
        var option = select.options[optionIndex];
        if ( option.text == cell.textContent ) {
          // We have found the value of this cell in the select.
          if ( option.selected == true ) {
            // It is one of the selected values,
            // so move on to the next cell.
            continue;
          }
          else {
            // The value in this cell is not one of the selected values for this column,
            // so hide the row.
            row.style.visibility = 'hidden';
            return;
          }
        }
      }
    }
    // All the values are among the selected values, so show the row.
    row.style.visibility = 'visible';
  }// end of setRowDisplayed
  
  // When the selected values for one column change, we check all the columns in all the rows,
  // because the decision to display or hide a row depends on all the columns.
  function selectRowsToDisplay () {
    for ( var rowIndex = 0; rowIndex < editTable.rows.length; rowIndex++ ) {
      setRowDisplayed( editTable.rows[rowIndex] );
    }
  }// end of function selectRowsToDisplay
  function addSelectOnchange ( select ) {
    select.onchange = selectRowsToDisplay;
  }// end of addSelectOnchange
  selects.forEach( addSelectOnchange );
  
  // Every time we get a new message, we add it to the table.
  function addMessageRow ( message ) {
    var row = editTable.insertRow(AT_END);
    addCellAndOption( row, clientIdSelect, message, CLIENT_ID, null );
    addCellAndOption( row, actionSelect, message, ACTION, null );
    addCellAndOption( row, timestampSelect, message, SERVER_TIMESTAMP, makeTimeText );
    addCellAndOption( row, viewIdSelect, message, VIEW_ID, null );
    addCellAndOption( row, nodeIdSelect, message, NODE_ID, null );
    addCellAndOption( row, edgeIdSelect, message, EDGE_ID, null );
    addCellAndOption( row, userNameSelect, message, USER_NAME, null );
    addCellAndOption( row, textSelect, message, TEXT, null );
    addCellAndOption( row, urlSelect, message, URL, null );
    addCellAndOption( row, xSelect, message, X, null );
    addCellAndOption( row, ySelect, message, Y, null );
    addCellAndOption( row, subjectIdSelect, message, SUBJECT_ID, null );
    addCellAndOption( row, predicateIdSelect, message, PREDICATE_ID, null );
    addCellAndOption( row, objectIdSelect, message, OBJECT_ID, null );
    // Check whether or not to display the row.
    setRowDisplayed( row );
  }
  logMessageCallbacks.push(addMessageRow);
  
}// end of function setUpEditHistoryTable

function onLoadEditHistory () {
  
  setUpEditHistoryTable();
  startPrimus();
  
}// end of function onLoadEditHistory

function onUnloadEditHistory () {
  
  endPrimus();
  
}// end of function onUnloadEditHistory