
// Onload will set this to the root SVG container.
var svg = null;

// reminder that passing true to cloneNode tells it to make a deep clone:
var DEEPLY = true;

// our model of the graph:

function objectCoordinateToSVGCoordinate ( coord ) {
  return 500*coord;
}// end of function objectCoordinateToSVGCoordinate

function ViewSVG ( view ) {
  
  var id = view.getId();
  var viewContainer = document.getElementById('viewContainerTemplate').cloneNode(DEEPLY);
  viewContainer.id = id+'View';
  var title = document.getElementById('viewTitleTemplate').cloneNode(DEEPLY);
  title.id = id+'Title';
  title.textContent = view.getId();
  viewContainer.appendChild( title );
  var edgeContainer = document.getElementById('edgeContainerTemplate').cloneNode(DEEPLY);
  edgeContainer.id = id+'EdgeContainer';
  viewContainer.appendChild( edgeContainer );
  var nodeContainer = document.getElementById('nodeContainerTemplate').cloneNode(DEEPLY);
  nodeContainer.id = id+'NodeContainer';
  viewContainer.appendChild( nodeContainer );
  
  this.getElement = function () {
    return viewContainer;
  };// end of this.getElement
  
  this.appendNode = function ( nodeElement ) {
    nodeContainer.appendChild( nodeElement );
  };// end of this.appendNode
  
  this.removeNode = function ( nodeElement ) {
    nodeContainer.removeChild( nodeElement );
  };
  
  this.appendEdge = function ( edgeElement ) {
    edgeContainer.appendChild( edgeElement );
  };// end of this.appendEdge
  
  this.removeEdge = function ( edgeElement ) {
    edgeContainer.removeChild( edgeElement );
  };// end of this.removeEdge
  
}// end of function ViewSVG

function NodeSVG ( node ) {
  
  var id = node.getId();
  var clientXOffset = 0.01;
  var clientYOffset = 0.05;
  
  var nodeRoot = document.getElementById('nodeTemplate').cloneNode(DEEPLY);
  nodeRoot.id = id+'Node';
  var link = document.getElementById('linkTemplate').cloneNode(DEEPLY);
  link.id = id+'Link';
  nodeRoot.appendChild( link );
  // var image = document.getElementById('imageTemplate').cloneNode(DEEPLY);
  // image.id = id+'Image';
  var text = document.getElementById('textTemplate').cloneNode(DEEPLY);
  text.id = id+'Text';
  link.appendChild( text );
  var client = document.getElementById('clientTemplate').cloneNode(DEEPLY);
  client.id = id+'Client';
  nodeRoot.appendChild( client );
  
  text.textContent = node.textWatch.get();
  function updateText ( oldText, newText ) {
    text.textContent = newText;
  }// end of function updateText
  node.textWatch.addChangeCallback(updateText);
  
  link.setAttributeNS( 'http://www.w3.org/1999/xlink', 'xlink:href', node.urlWatch.get() );
  function updateUrl ( oldUrl, newUrl ) {
    link.setAttributeNS( 'http://www.w3.org/1999/xlink', 'xlink:href', newUrl );
  }// end of function updateUrl
  node.urlWatch.addChangeCallback(updateUrl);
  
  var currentClientObject = node.clientWatch.get();
  client.textContent = ( currentClientObject !== null ) ? currentClientObject.getId()+" editing" : "";
  function updateClient ( oldClient, newClient ) {
    client.textContent = ( newClient !== null ) ? newClient.getId()+" editing" : "";
  }// end of function updateClient 
  node.clientWatch.addChangeCallback(updateClient);
  
  text.setAttributeNS(  null,  'x',  objectCoordinateToSVGCoordinate( node.xWatch.get() )  );
  client.setAttributeNS(  null,  'x',  objectCoordinateToSVGCoordinate( node.xWatch.get() + clientXOffset )  );
  function updateX ( oldX, newX ) {
    text.setAttributeNS(  null,  'x',  objectCoordinateToSVGCoordinate( newX )  );
    client.setAttributeNS(  null,  'x',  objectCoordinateToSVGCoordinate( newX + clientXOffset )  );
  }// end of function updateX
  node.xWatch.addChangeCallback(updateX);
  
  text.setAttributeNS(  null,  'y',  objectCoordinateToSVGCoordinate( node.yWatch.get() )  );
  client.setAttributeNS(  null,  'y',  objectCoordinateToSVGCoordinate( node.yWatch.get() + clientYOffset )  );
  function updateY ( oldY, newY ) {
    text.setAttributeNS(  null,  'y',  objectCoordinateToSVGCoordinate( newY )  );
    client.setAttributeNS(  null,  'y',  objectCoordinateToSVGCoordinate( newY + clientYOffset )  );
  }// end of function updateY
  node.yWatch.addChangeCallback(updateY);
  
  this.getElement = function () {
    return nodeRoot;
  };// end of this.getElement
  
}// end of function NodeSVG

function EdgeSVG ( edge ) {
  
  var edgeRoot = document.getElementById('edgeTemplate').cloneNode(DEEPLY);
  edgeRoot.id = edge.getId()+'Edge';
  
  var ARROWHEAD_ANGLE = Math.PI/6;// how far from the shaft to put the prongs
  var HEAD_LENGTH = 0.05;// the length of the arrowhead parts
  
  // To represent an edge, we make a quadratic Bezier curve
  // starting a short distance from sorc
  // and ending in an arrowhead a short distance from dest
  function setPath () {
    var sorcX = edge.subjectWatch.get().xWatch.get();
    var sorcY = edge.subjectWatch.get().yWatch.get();
    var lablX = edge.predicateWatch.get().xWatch.get();
    var lablY = edge.predicateWatch.get().yWatch.get();
    var destX = edge.objectWatch.get().xWatch.get();
    var destY = edge.objectWatch.get().yWatch.get();
    var midlX = 2*lablX - (sorcX + destX)/2;
    var midlY = 2*lablY - (sorcY + destY)/2;
    var headAngleBase = Math.atan2( (destY - midlY), (destX - midlX) );
    var headAngle1 = headAngleBase + ARROWHEAD_ANGLE;
    var headAngle2 = headAngleBase - ARROWHEAD_ANGLE;
    var headX1 = destX - HEAD_LENGTH*Math.cos(headAngle1);
    var headY1 = destY - HEAD_LENGTH*Math.sin(headAngle1);
    var headX2 = destX - HEAD_LENGTH*Math.cos(headAngle2);
    var headY2 = destY - HEAD_LENGTH*Math.sin(headAngle2);
    var x1 = objectCoordinateToSVGCoordinate(sorcX);
    var y1 = objectCoordinateToSVGCoordinate(sorcY);
    var x2 = objectCoordinateToSVGCoordinate(midlX);
    var y2 = objectCoordinateToSVGCoordinate(midlY);
    var x3 = objectCoordinateToSVGCoordinate(destX);
    var y3 = objectCoordinateToSVGCoordinate(destY);
    var x4 = objectCoordinateToSVGCoordinate(headX1);
    var y4 = objectCoordinateToSVGCoordinate(headY1);
    var x5 = objectCoordinateToSVGCoordinate(headX2);
    var y5 = objectCoordinateToSVGCoordinate(headY2);
    // M: move to (without drawing anything)
    // L: line to
    // Q: quadratic Bezier curve with control point and second end point
    var d = 'M '+x1+' '+y1+' Q '+x2+' '+y2+' '+x3+' '+y3+' L '+x4+' '+y4+' M '+x3+' '+y3+' L '+x5+' '+y5;
    edgeRoot.setAttributeNS(null,'d',d);
  };// end of function setPath
  
  setPath();
  
  // Since we need to get the coordinates from all three nodes to set the path,
  // we call the same function that recomputes the whole thing any time one point changes its x or y coordinate.
  edge.subjectWatch.get().xWatch.addChangeCallback(setPath);
  edge.subjectWatch.get().yWatch.addChangeCallback(setPath);
  edge.predicateWatch.get().xWatch.addChangeCallback(setPath);
  edge.predicateWatch.get().yWatch.addChangeCallback(setPath);
  edge.objectWatch.get().xWatch.addChangeCallback(setPath);
  edge.objectWatch.get().yWatch.addChangeCallback(setPath);
  
  this.getElement = function () {
    return edgeRoot;
  };// end of this.getElement
  
}// end of fuction EdgeSVG

var viewSVGs = {};
var nodeSVGs = {};
var edgeSVGs = {};

// We should only look at one view element at a time.
// Use these to keep track of which view we are seeing.
var viewIdArray = [];
var currentViewIndex = 0;
var currentViewId = null;
var currentViewSVG = null;

function advanceViewSVG () {
  // Deattach the current view element.
  svg.removeChild( currentViewSVG.getElement() );
  // Attach the next one.
  currentViewIndex = ( currentViewIndex+1 ) % ( viewIdArray.length );
  currentViewId = viewIdArray[currentViewIndex];
  currentViewSVG = viewSVGs[currentViewId];
  svg.appendChild( currentViewSVG.getElement() );
  sendUpdateClientView( currentViewId );
}// end of function advanceViewSVG

function insertView ( id, view ) {
  var viewSVG = new ViewSVG(view);
  viewSVGs[id] = viewSVG;
  viewIdArray.push(id);
  // The current view is null if and only if this view is the first view added.
  if ( currentViewSVG === null ) {
    currentViewSVG = viewSVG;
    svg.appendChild( viewSVG.getElement() );
    currentViewIndex = viewIdArray.length-1;
    sendUpdateClientView( id );
  }
}// end of function insertView

function removeView ( id, view ) {
  // Only change what we are viewing if the view we are viewing now is the one being deleted.
  if ( currentViewId === id ) {
    if ( viewIdArray.length > 1 ) {
      // Move on to the next view element if there is one.
      advanceViewSVG();
    }
    else {
      // In the special case where we have no more view elements,
      // remove the view without trying to show another.
      svg.removeChild( viewSVGs[id].getElement() );
      currentViewIndex = 0;
      currentViewId = null;
      currentViewSVG = null;
      sendUpdateClientView( null );
    }
  }
  viewIdArray.splice( viewIdArray.indexOf(id), 1 );
  delete viewSVGs[id];
}// end of function removeView

function insertNode ( id, node ) {
  var nodeSVG = new NodeSVG(node);
  nodeSVGs[id] = nodeSVG;
  viewId = node.viewWatch.get().getId();
  viewSVGs[viewId].appendNode( nodeSVG.getElement() );
}// end of function insertNode

function removeNode ( id, node ) {
  viewId = node.viewWatch.get().getId();
  viewSVGs[viewId].removeNode( nodeSVGs[id].getElement() );
  delete nodeSVGs[id];
}// end of function removeNode

function insertEdge ( id, edge ) {
  var edgeSVG = new EdgeSVG(edge);
  edgeSVGs[id] = edgeSVG;
  viewId = edge.viewWatch.get().getId();
  viewSVGs[viewId].appendEdge( edgeSVG.getElement() );
}// end of function insertEdge

function removeEdge ( id, edge ) {
  viewId = edge.viewWatch.get().getId();
  viewSVGs[viewId].removeEdge( edgeSVGs[id].getElement() );
  delete edgeSVGs[id];
}// end of function removeEdge

function onLoadView () {
  svg = document.getElementById('svg');
  viewMap.addCreateCallback( insertView );
  viewMap.addDeleteCallback( removeView );
  nodeMap.addCreateCallback( insertNode );
  nodeMap.addDeleteCallback( removeNode );
  edgeMap.addCreateCallback( insertEdge );
  edgeMap.addDeleteCallback( removeEdge );
  startPrimus();
}// end of function onLoadView

function onBeforeUnloadView () {
  endPrimus();
}// end of function onBeforeUnloadView
