'use strict';

var dispatcher = require('./dispatcher');
var primusSetup = require('./primus-setup');
var backlog = require('./backlog');
var clientLibraryPath = 'client/primus.js';

var primusServer = null;

var ACTION = 'action';
var START_INDEX = 'startIndex';

var clients = [];

function onNewClient ( clientId ) {
  var message = new Object();
  message[ACTION] = 'createClient';
  message['serverTimestamp'] = Date.now();
  message['clientId'] = clientId;
  message['userName'] = 'anonymous';
  // Do not send the createClient message to the client now.
  // The client will get it when we send the backlog.
  // Sending it now would cause the client to receive it twice.
  backlog.save(message);
  function queueOutgoing (recipientId) {
    dispatcher.queueOutgoing(recipientId, message);
  }// end of function queueOutgoing
  clients.forEach(queueOutgoing);
  dispatcher.dispatchOutgoing();
  clients.push(clientId);
}// end of function onNewClient

function onIncomingMessage ( senderId, message ) {
  function queueOutgoing (recipientId) {
    dispatcher.queueOutgoing(recipientId, message);
  }// end of function queueOutgoing
  if (  ( message.hasOwnProperty(ACTION) )  &&  ( message[ACTION] === 'getBacklog' )  ) {
    // If the client is requesting the backlog of messages, return it.
    // Return all messages starting at the specified index or, if none, from the start.
    var startIndex = 0;
    if ( message.hasOwnProperty(START_INDEX) ) {
      startIndex = message[START_INDEX];
    }
    message['backlog'] = backlog.get(startIndex);
    dispatcher.queueOutgoing( senderId, message );
  }
  else if (  ( message.hasOwnProperty(ACTION) )  &&  ( message[ACTION] === 'getMyClientId' )  ) {
    // If the client is requesting its own client ID, send the message back to the client with the ID.
    message['myClientId'] = senderId;
    dispatcher.queueOutgoing( senderId, message );
  }
  else if (  ( message.hasOwnProperty(ACTION) )  &&  ( message[ACTION] === 'setFilter' )  ) {
    // If the client is trying to set the filter, ignore this message.
    // We have not implemented filtering yet.
  }
  else {
    // Relay messages of all other types to the other clients.
    message['serverTimestamp'] = Date.now();
    message['clientId'] = senderId;
    backlog.save(message);
    clients.forEach( queueOutgoing );
  }
  // Now that we have cued the messages, send them out.
  dispatcher.dispatchOutgoing();
}// end of function onIncomingMessage

function onClosedClient ( clientId ) {
  clients.splice( clients.indexOf(clientId), 1 );
  var message = new Object();
  message[ACTION] = 'deleteClient';
  message['serverTimestamp'] = Date.now();
  message['clientId'] = clientId;
  backlog.save(message);
  function queueOutgoing (recipientId) {
    dispatcher.queueOutgoing(recipientId, message);
  }// end of function queueOutgoing
  clients.forEach(queueOutgoing);
  dispatcher.dispatchOutgoing();
}// end of function onClosedClient

function onBacklogLoaded ( err, messages ) {
  if ( err ) {
    console.log( "failed to load saved messages: ", err);
    return;
  }
  primusServer = primusSetup.getPrimusServer(clientLibraryPath);
  dispatcher.init( primusServer, onNewClient, onIncomingMessage, onClosedClient );
  console.log('mindmap-node server ready');
}// end of function onBacklogLoaded

exports.init = function ( clientLibraryPath ) {
  backlog.load( 'messages.json', onBacklogLoaded );
};// end of exports.init

exports.end = function () {
  dispatcher.end();
  primusServer.end();
};// end of exports.end